#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@Project ：deep-head-pose-master 
@File ：fd_face_detection.py
@IDE ：PyCharm 
@Author ：zjj
@Date ：2023/9/12 13:38 
"""
import fastdeploy as fd
import cv2
from loguru import logger
from config import SCRFD_PATH_DICT, INSIGHTFACE_PATH_DICT, FACE_THRESHOLD
from utils import cosine_similarity


class FaceRecognition:
    """
    人脸识别类
    """

    def __init__(
        self,
        id_model=INSIGHTFACE_PATH_DICT["onnx"],
        det_model=SCRFD_PATH_DICT["onnx"],
        device="gpu",
        use_trt=False,
        half=False,
    ):
        """
        初始化相机人脸检测器
        :param id_model: 人脸识别模型
        :param det_model: 人脸检测模型
        :param device: 使用设备"cpu"/"gpu"
        :param use_trt: 是否使用tenserRT加速
        :param half: 是否半精度推理
        """
        self.id_model = id_model
        self.det_model = det_model
        self.device = device
        self.use_trt = use_trt
        self.half = half

        # 用统一的方法加载模型
        self.det_model = self.load_model(fd.vision.facedet.SCRFD, det_model, "SCRFD")
        self.id_model = self.load_model(
            fd.vision.faceid.PartialFC, id_model, "PartialFC"
        )

    def load_model(self, model_class, model_path, model_type):
        """
        加载模型
        :param model_class: 实例化模型类
        :param model_path: 模型路径
        :param model_type: 模型类型
        :return:
        """
        runtime_option = self.set_option(model_type)
        model = model_class(model_path, runtime_option=runtime_option)
        return model

    @staticmethod
    def extract_face(frame, box, padding=10):
        """
        从图像帧中提取人脸图像
        :param frame: 输入的图像帧
        :param box: 人脸检测框
        :padding: 放大box大小
        :return: 提取的人脸图像
        """
        # 提取人脸检测框的边界坐标
        # x_min, y_min, x_max, y_max = map(int, box)

        # 提取人脸检测框的边界坐标，并添加额外的空白区域
        x_min, y_min, x_max, y_max = map(
            int,
            [box[0] - padding, box[1] - padding, box[2] + padding, box[3] + padding],
        )

        # 确保人脸图像的提取范围不超出原始图像边界
        x_min = max(0, x_min)
        x_max = min(frame.shape[1], x_max)
        y_min = max(0, y_min)
        y_max = min(frame.shape[0], y_max)

        # 从原始图像中提取人脸图像
        face_image = frame[y_min:y_max, x_min:x_max]
        face_image = cv2.resize(
            face_image, (112, 112), interpolation=cv2.INTER_AREA
        )  # 将人脸缩放到人脸识别模型默认的大小

        return face_image

    def set_option(self, model_type):
        """
        构建运行时选项
        :return: option
        """
        option = fd.RuntimeOption()
        if self.device.lower() == "gpu":
            option.use_gpu()
        if self.use_trt:
            option.use_trt_backend()
            if self.half:
                option.trt_option.enable_fp16 = True

            if model_type == "PartialFC":
                option.set_trt_input_shape("data", [1, 3, 112, 112])  # fd默认参数
                option.set_trt_cache_file(INSIGHTFACE_PATH_DICT["trt"])
            if model_type == "SCRFD":
                option.set_trt_input_shape("images", [1, 3, 640, 640])
                option.set_trt_cache_file(SCRFD_PATH_DICT["trt"])

        return option

    def get_img_embedding(self, img):
        """
        用模型为img编码
        :param img:
        :return: list(512)
        """
        img_copy = img.copy()
        base_face = self.id_model.predict(img_copy)  # 基准照
        embedding = base_face.embedding
        return embedding

    def face_compare(self, base_img, current_img):
        """
        人脸识别
        :param base_img: 基准照
        :param current_img: 要对比的当前考生照
        :return: 相似度值
        """
        # todo id_model会改变输入的参数！！！
        embed1 = self.get_img_embedding(base_img)
        embed2 = self.get_img_embedding(current_img)

        cos = cosine_similarity(embed1, embed2)
        return cos

    def get_crop_face(self, frame):
        # 检测人脸 todo conf_threshold default 0.7
        result = self.det_model.predict(frame, conf_threshold=0.7)
        if result and result.boxes and result.scores[0] > FACE_THRESHOLD:
            # 裁剪后的基准照 不管多少个人脸，只选取第一个
            base_face = self.extract_face(frame, result.boxes[0])
            return True, base_face
        else:
            logger.warning("无法检测到人脸")
            return False, None

    # def get_target_face(self, frame, p1=None, p2=None):
    #     result = self.det_model.predict(frame)
    #     if not result:
    #         return False, None
    #     imh, imw, _ = frame.shape
    #     min_dis = float('inf')
    #     target_box = []
    #     for box, score in zip(result.boxes, result.scores):
    #         if score > FACE_THRESHOLD:
    #             xmin, ymin, xmax, ymax = box
    #             if p1 and p2:
    #                 if xmin >= p1[0] and ymin >= p1[1] and xmax <= p2[0] and ymax <= p2[1]:
    #                     box_x_mid = xmin + (xmax - xmin) / 2
    #                     # 多个人脸时选取离中心轴最近的人脸
    #                     dis = abs(box_x_mid - imw / 2)
    #                     if dis < min_dis:
    #                         target_box = box
    #     if target_box:
    #         base_face = self.extract_face(frame, target_box)
    #         return True, base_face
    #     return False, None

    def get_target_face(self, frame, mask=None):
        """

        :param frame:
        :param mask: 目标是1，其余是0
        :return:
        """
        result = self.det_model.predict(frame)
        if not result:
            return False, None
        imh, imw, _ = frame.shape
        target_box = []
        for box, score in zip(result.boxes, result.scores):
            if score > FACE_THRESHOLD:
                xmin, ymin, xmax, ymax = box
                box_x_mid = int(xmin + (xmax - xmin) / 2)
                box_y_mid = int(ymin + (ymax - ymin) / 2)
                # 人脸框中心点在目标范围内
                if mask[box_y_mid, box_x_mid] > 0:
                    target_box = box
        if target_box:
            base_face = self.extract_face(frame, target_box)
            return True, base_face
        return False, None


if __name__ == "__main__":
    # faceid = FaceRecognition(device="cpu", use_trt=False)
    faceid = FaceRecognition(
        id_model=r"/home/user/zhoujinjuan/models/partial_fc_glint360k_r100.onnx",
        det_model=r"/home/user/zhoujinjuan/models/scrfd_500m_bnkps_shape640x640.onnx",
        device="cpu",
        use_trt=False,
    )
    img = cv2.imread('data/11.png')
    embed = faceid.get_img_embedding(img)
    has_face, face = faceid.get_crop_face(img)
    if has_face:
        cv2.imwrite('data/face.png', face)
    else:
        print('no face')

