#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@Project ：yolo_demos 
@File ：main.py
@IDE ：PyCharm 
@Author ：zjj
@Date ：2023/9/26 9:29 
"""
import numpy as np
from loguru import logger
from ultralytics import YOLO

from config import (
    PERSON,
    CAR,
    SEGMENT_MODEL,
    TOPN_AREA,
    LABEL_NAMES,
    BOX_AREA_THRESHOLD,
    DETECT_CONF_THRESHOLD,
    ARTICULATION_THD, MOTOR_VEHICLE, MOTOR_VEHICLE_LIST,
)
from fd_face_detection import FaceRecognition
from fd_yolo import FdYolov8
from utils import (
    extract_yolo_results,
    get_police,
    predict_ocr,
    get_target_mask,
    draw_rectangle_text,
    get_coordinate,
    det_articulation,
)


class TrackMain(object):
    def __init__(self, detect_model_path=SEGMENT_MODEL, device="gpu", use_trt=False, half=False):
        # 目标检测模型
        self.yolo_seg_model = YOLO(detect_model_path)
        # 人脸检测和识别对象
        self.face_det = FaceRecognition(device=device, use_trt=use_trt, half=half)
        # 交警模型
        self.model_traffic_police = FdYolov8(device=device, use_trt=use_trt, half=half)

    def process_one_frame(self, frame):
        """
        处理一帧数据
        :param frame:
        :return:
        """
        results = self.yolo_seg_model(frame)

        # 提取模型检测结果 infos=[xywhs, cls, scores, xy_list]
        is_hit, infos = extract_yolo_results(results)
        if not is_hit:
            return [], []

        # 不统计频率，大框筛选之后，设置TOPN过滤，不想过滤则可以调整N值很大
        # 面积和置信度过滤，统计剩余目标的相关信息：左上角和右下角坐标,label,conf,box_area,轮廓坐标
        tinfos_each_frame = self.statistics_one_frame(infos)

        # annotated_frame = results[0].plot()
        face_embeddings, licenses, _ = self.process_topn_in_one_frame(frame, tinfos_each_frame)
        return face_embeddings, licenses

    @classmethod
    def statistics_one_frame(cls, infos):
        """
        每一帧内统计每个目标的信息
        :param infos:
        :return:
        """
        xywhs, labels, scores, xy_list = infos
        tinfos_each_frame = {PERSON: [], MOTOR_VEHICLE: []}

        for xywh, label_id, score, polygon in zip(
            xywhs, labels, scores, xy_list
        ):
            if label_id not in list(LABEL_NAMES.keys()):
                continue

            # 每个目标的信息
            info = {'p_left_up': None, 'p_right_bottom': None, 'label': None, 'conf': None, 'box_area': None,
                    'polygon_indexs': None}

            x, y, w, h = xywh
            # 根据xywh获取左上角和右下角坐标
            p1, p2 = get_coordinate(xywh)
            # 计算框面积
            s = int((w * h) / 10000)
            label = LABEL_NAMES[label_id]
            conf = round(score, 2)

            # 通过面积过滤掉一些小框
            # todo 预筛选，阈值不能太高，后面还会有topN过滤
            if s <= BOX_AREA_THRESHOLD:
                continue
            if conf <= DETECT_CONF_THRESHOLD:
                continue
            # 记录该类别下该目标的相关信息
            info['p_left_up'] = p1
            info['p_right_bottom'] = p2
            info['label'] = label
            info['conf'] = conf
            info['box_area'] = s
            info['polygon_indexs'] = polygon.astype(int)
            # 机动车包含car,truck,motorcycle,bus
            if label in MOTOR_VEHICLE_LIST:
                tinfos_each_frame[MOTOR_VEHICLE].append(info)
            else:
                tinfos_each_frame[label].append(info)

        return tinfos_each_frame

    def process_topn_in_one_frame(self, frame, tinfos_each_frame):
        """
        处理每一帧中的topn大框对象:
        person类：先用交警模型过滤掉交警，获取目标的人脸，根据清晰度过滤，对保留下来的人脸提取embedding；
        car类：ocr识别车牌号，每一帧中车牌号去重；
        :param frame:
        :param tinfos_each_frame: 每个类别下各个目标的相关信息
        :return:
        """
        frame_copy = frame.copy()

        police_indexs = []
        # 存储一帧中所有车牌号
        all_licenses = []
        # 存储一帧中所有人脸embedding
        all_face_embeddings = []
        for label_alias, target_infos in tinfos_each_frame.items():
            # 按照框面积大小降序排序
            target_infos.sort(key=lambda x: x['box_area'], reverse=True)

            # todo 交警 先对整个图片上前topn的大框图中的person，送到交警检测模型中检测是否是交警
            if label_alias == PERSON:
                police_indexs = get_police(
                    frame_copy, target_infos[:TOPN_AREA], self.model_traffic_police
                )

            # 每帧每个类别显示top2大的目标
            for index, info in enumerate(target_infos[:TOPN_AREA]):
                is_hit = False
                # label = info['label']
                p1 = info['p_left_up']
                p2 = info['p_right_bottom']
                polygon = info['polygon_indexs']
                # 直接用目标检测的框
                target_img = frame[p1[1]: p2[1], p1[0]: p2[0]]
                target_img = target_img.astype(np.uint8)

                if label_alias == MOTOR_VEHICLE:
                    # licenses = predict_ocr(target_img, self.ocr)
                    licenses = predict_ocr(target_img)
                    licenses = list(set(licenses))
                    all_licenses.extend(licenses)
                    if licenses:
                        is_hit = True

                elif label_alias == PERSON:
                    # 是交警，则不处理
                    if index in police_indexs:
                        continue
                    # 二值图像，人-1， 背景-0
                    target_mask = get_target_mask(frame, polygon)
                    # todo 对整个图检测人脸可能得到多个人脸，但只对target_img的白底目标图检测人脸，很可能无法检测到人脸，因此结合实例mask定位目标人脸
                    has_face, face = self.face_det.get_target_face(frame, target_mask)
                    if has_face:
                        # 用人脸清晰度过滤
                        score = det_articulation(face)
                        # todo 阈值大小多少合适？？
                        logger.debug(f'人脸清晰度{score}')
                        if score < ARTICULATION_THD:
                            continue
                        face_embedding = self.face_det.get_img_embedding(face)
                        all_face_embeddings.append(face_embedding)
                        is_hit = True
                # test
                if is_hit:
                    frame_copy = draw_rectangle_text(
                        frame_copy, index, p1, p2, label_alias, info['conf'], -1, info['box_area']
                    )
        return all_face_embeddings, all_licenses, frame_copy

